'use strict';

var fs = require('fs');
var path = require('path');
var prompts = require('prompts');
var kolorist = require('kolorist');
var url = require('url');
var nanospinner = require('nanospinner');
var child_process = require('child_process');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var prompts__default = /*#__PURE__*/_interopDefaultLegacy(prompts);

const FRAMEWORKS = [
    {
        name: 'vanilla',
        display: 'Vanilla',
        color: kolorist.yellow,
        variants: [
            {
                name: 'vanilla',
                display: 'JavaScript',
                color: kolorist.yellow
            },
            {
                name: 'vanilla-ts',
                display: 'TypeScript',
                color: kolorist.blue
            }
        ]
    },
    {
        name: 'vue',
        display: 'Vue',
        color: kolorist.green,
        variants: [
            {
                name: 'vue',
                display: 'JavaScript',
                color: kolorist.yellow
            },
            {
                name: 'vue-ts',
                display: 'TypeScript',
                color: kolorist.blue
            }
        ]
    },
    {
        name: 'react',
        display: 'React',
        color: kolorist.cyan,
        variants: [
            {
                name: 'react',
                display: 'JavaScript',
                color: kolorist.yellow
            },
            {
                name: 'react-ts',
                display: 'TypeScript',
                color: kolorist.blue
            }
        ]
    }
];
FRAMEWORKS.map((f) => (f.variants && f.variants.map((v) => v.name)) || [f.name]).reduce((a, b) => a.concat(b), []);

const root = process.cwd();
function pkgFromUserAgent(userAgent) {
    if (!userAgent)
        return undefined;
    const pkgSpec = userAgent.split(' ')[0];
    const pkgSpecArr = pkgSpec.split('/');
    return {
        name: pkgSpecArr[0],
        version: pkgSpecArr[1]
    };
}
const getPackageManager = () => {
    const pkgInfo = pkgFromUserAgent(process.env.npm_config_user_agent);
    return pkgInfo ? pkgInfo.name : 'pnpm';
};
function copy(src, dest) {
    const stat = fs__default["default"].statSync(src);
    if (stat.isDirectory()) {
        copyDir(src, dest);
    }
    else {
        fs__default["default"].copyFileSync(src, dest);
    }
}
function copyDir(srcDir, destDir) {
    fs__default["default"].mkdirSync(destDir, { recursive: true });
    for (const file of fs__default["default"].readdirSync(srcDir)) {
        const srcFile = path__default["default"].resolve(srcDir, file);
        const destFile = path__default["default"].resolve(destDir, file);
        copy(srcFile, destFile);
    }
}
const askForProjectLint = async () => {
    let result;
    try {
        result = await prompts__default["default"]([
            {
                type: 'select',
                name: 'framework',
                message: kolorist.reset('Select a framework:'),
                initial: 0,
                choices: FRAMEWORKS.map((framework) => {
                    const frameworkColor = framework.color;
                    return {
                        title: frameworkColor(framework.display || framework.name),
                        value: framework
                    };
                })
            },
            {
                type: (framework) => (framework && framework.variants ? 'select' : null),
                name: 'variant',
                message: kolorist.reset('Select a variant:'),
                choices: (framework) => framework.variants.map((variant) => {
                    const variantColor = variant.color;
                    return {
                        title: variantColor(variant.display || variant.name),
                        value: variant.name
                    };
                })
            },
            {
                type: 'multiselect',
                name: 'otherLint',
                message: kolorist.underline('Pick other lint'),
                choices: [
                    { title: 'commitlint', value: 'commitlint' },
                    { title: 'stylelint', value: 'stylelint' }
                ],
                hint: '- Space to select. Return to submit'
            },
            {
                type: 'confirm',
                name: 'isVscode',
                message: kolorist.underline('loading default vscode setting?'),
                initial: true
            }
        ], {
            onCancel: () => {
                throw new Error(kolorist.red('✖') + ' Operation cancelled');
            }
        });
        return result;
    }
    catch (cancelled) {
        console.log(cancelled.message);
        return;
    }
};
const write = (file, content) => {
    const targetPath = path__default["default"].join(root, file);
    if (content) {
        fs__default["default"].writeFileSync(targetPath, content);
    }
    else {
        copy(path__default["default"].join(root, file), targetPath);
    }
};
// TODO: add husky config and lint-stage config
const generatePackageJson = ({ otherLint, variant }) => {
    const huskyConfig = {
        hooks: {
            'pre-commit': 'lint-staged'
        }
    };
    const lintStagedConfig = {
        '*.{js,jsx,ts,tsx}': ['eslint --fix', 'prettier --write'],
        '{!(package)*.json,*.code-snippets,.!(browserslist)*rc}': ['prettier --write--parser json'],
        'package.json': ['prettier --write'],
        '*.md': ['prettier --write']
    };
    let scripts = {
        'lint:eslint': 'eslint --cache --max-warnings 0  "{src,mock}/**/*.{vue,ts,tsx}" --fix',
        'lint:prettier': 'prettier --write  "src/**/*.{js,json,tsx,css,less,scss,vue,html,md}"',
        'lint:lint-staged': 'lint-staged',
        prepare: 'husky install'
    };
    if (otherLint.includes('stylelint')) {
        lintStagedConfig['*.{scss,less,styl}'] = ['stylelint --fix', 'prettier --write'];
        scripts['lint:stylelint'] =
            'stylelint --cache --fix "**/*.{vue,less,postcss,css,scss}" --cache --cache-location node_modules/.cache/stylelint/';
    }
    if (variant.includes('vue')) {
        if (otherLint.includes('stylelint')) {
            lintStagedConfig['*.vue'] = ['eslint --fix', 'prettier --write', 'stylelint --fix'];
        }
        else {
            lintStagedConfig['*.vue'] = ['eslint --fix', 'prettier --write'];
        }
    }
    const pkg = JSON.parse(fs__default["default"].readFileSync(path__default["default"].join(root, `package.json`), 'utf-8'));
    pkg.scripts = {
        ...pkg.scripts,
        ...scripts
    };
    pkg['lint-staged'] = {
        ...pkg['lint-staged'],
        ...lintStagedConfig
    };
    pkg['husky'] = {
        ...pkg['husky'],
        ...huskyConfig
    };
    write('package.json', JSON.stringify(pkg, null, 2));
};
const writeJsonFile = (fileName, json) => {
    const filePath = path__default["default"].join(root, fileName);
    fs__default["default"].writeFileSync(filePath, JSON.stringify(json, null, 2));
};
const checkPackageFile = () => {
    if (!fs__default["default"].existsSync(path__default["default"].join(root, 'package.json'))) {
        console.log(kolorist.red('✖') + 'no such file or directory: package.json');
        return false;
    }
    return true;
};

const commonPackages = [
    'husky',
    'lint-staged',
    'eslint',
    'prettier',
    'eslint-plugin-prettier',
    'eslint-config-prettier'
];
const commitlintPackages = [
    '@commitlint/cli',
    '@commitlint/config-conventional',
    'commitizen',
    'commitlint-config-cz'
];
const eslintPackages = {
    vanilla: [],
    'vanilla-ts': ['@typescript-eslint/eslint-plugin', '@typescript-eslint/parser'],
    vue: ['vue-eslint-parser', 'eslint-plugin-vue'],
    'vue-ts': [
        '@typescript-eslint/eslint-plugin',
        '@typescript-eslint/parser',
        'vue-eslint-parser',
        'eslint-plugin-vue'
    ],
    react: ['eslint-plugin-react'],
    'react-ts': [
        '@typescript-eslint/eslint-plugin',
        '@typescript-eslint/parser',
        'eslint-plugin-react'
    ]
};

const eslintConfig = {
    env: {
        browser: true,
        es2021: true,
        node: true
    },
    overrides: []
};
const eslintOverrides = {
    vanilla: [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        }
    ],
    'vanilla-ts': [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.ts'],
            parser: '@typescript-eslint/parser',
            parserOptions: {
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:prettier/recommended'
            ]
        }
    ],
    vue: [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.vue'],
            parser: 'vue-eslint-parser',
            parserOptions: {
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: ['eslint:recommended', 'plugin:vue/vue3-recommended', 'plugin:prettier/recommended'],
            rules: {
                'vue/multi-word-component-names': 'off'
            }
        }
    ],
    'vue-ts': [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.ts'],
            parser: '@typescript-eslint/parser',
            parserOptions: {
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:prettier/recommended'
            ]
        },
        {
            files: ['*.vue'],
            parser: 'vue-eslint-parser',
            parserOptions: {
                parser: '@typescript-eslint/parser',
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:vue/vue3-recommended',
                'plugin:prettier/recommended'
            ],
            rules: {
                'vue/multi-word-component-names': 'off'
            }
        }
    ],
    react: [
        {
            files: ['*.js', '*.jsx'],
            parserOptions: {
                ecmaFeatures: {
                    jsx: true
                },
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: ['eslint:recommended', 'plugin:react/recommended', 'plugin:prettier/recommended'],
            rules: {
                'react/react-in-jsx-scope': 'off'
            }
        }
    ],
    'react-ts': [
        {
            files: ['*.js'],
            extends: ['eslint:recommended', 'plugin:prettier/recommended']
        },
        {
            files: ['*.ts', '*.tsx'],
            parser: '@typescript-eslint/parser',
            parserOptions: {
                ecmaFeatures: {
                    jsx: true
                },
                ecmaVersion: 'latest',
                sourceType: 'module'
            },
            extends: [
                'eslint:recommended',
                'plugin:@typescript-eslint/recommended',
                'plugin:react/recommended',
                'plugin:prettier/recommended'
            ],
            rules: {
                'react/react-in-jsx-scope': 'off'
            }
        }
    ]
};
const getEslintConfig = (variant) => {
    return { ...eslintConfig, overrides: eslintOverrides[variant] };
};
const stylelintPackages = [
    'stylelint',
    'stylelint-config-prettier',
    'stylelint-config-rational-order',
    'stylelint-config-standard',
    'stylelint-order'
];
const getStylelintPackages = (variant) => {
    if (variant.includes('vue')) {
        return [...stylelintPackages, 'postcss-html', 'stylelint-config-recommended-vue'];
    }
    return stylelintPackages;
};
const stylelintConfig = {
    root: true,
    plugins: ['stylelint-order'],
    extends: ['stylelint-config-standard'],
    rules: {
        'selector-pseudo-class-no-unknown': [
            true,
            {
                ignorePseudoClasses: ['global']
            }
        ],
        'selector-pseudo-element-no-unknown': [
            true,
            {
                ignorePseudoElements: ['v-deep']
            }
        ],
        'at-rule-no-unknown': [
            true,
            {
                ignoreAtRules: ['function', 'if', 'each', 'include', 'mixin']
            }
        ],
        'no-empty-source': null,
        'named-grid-areas-no-invalid': null,
        'unicode-bom': 'never',
        'no-descending-specificity': null,
        'font-family-no-missing-generic-family-keyword': null,
        'declaration-colon-space-after': 'always-single-line',
        'declaration-colon-space-before': 'never',
        'declaration-block-trailing-semicolon': 'always',
        'rule-empty-line-before': [
            'always',
            {
                ignore: ['after-comment', 'first-nested']
            }
        ],
        'unit-no-unknown': [true, { ignoreUnits: ['rpx'] }],
        'order/order': [
            [
                'dollar-variables',
                'custom-properties',
                'at-rules',
                'declarations',
                {
                    type: 'at-rule',
                    name: 'supports'
                },
                {
                    type: 'at-rule',
                    name: 'media'
                },
                'rules'
            ],
            { severity: 'warning' }
        ],
        'order/properties-order': [
            'position',
            'top',
            'right',
            'bottom',
            'left',
            'z-index',
            'display',
            'float',
            'width',
            'height',
            'max-width',
            'max-height',
            'min-width',
            'min-height',
            'padding',
            'padding-top',
            'padding-right',
            'padding-bottom',
            'padding-left',
            'margin',
            'margin-top',
            'margin-right',
            'margin-bottom',
            'margin-left',
            'margin-collapse',
            'margin-top-collapse',
            'margin-right-collapse',
            'margin-bottom-collapse',
            'margin-left-collapse',
            'overflow',
            'overflow-x',
            'overflow-y',
            'clip',
            'clear',
            'font',
            'font-family',
            'font-size',
            'font-smoothing',
            'osx-font-smoothing',
            'font-style',
            'font-weight',
            'hyphens',
            'src',
            'line-height',
            'letter-spacing',
            'word-spacing',
            'color',
            'text-align',
            'text-decoration',
            'text-indent',
            'text-overflow',
            'text-rendering',
            'text-size-adjust',
            'text-shadow',
            'text-transform',
            'word-break',
            'word-wrap',
            'white-space',
            'vertical-align',
            'list-style',
            'list-style-type',
            'list-style-position',
            'list-style-image',
            'pointer-events',
            'cursor',
            'background',
            'background-attachment',
            'background-color',
            'background-image',
            'background-position',
            'background-repeat',
            'background-size',
            'border',
            'border-collapse',
            'border-top',
            'border-right',
            'border-bottom',
            'border-left',
            'border-color',
            'border-image',
            'border-top-color',
            'border-right-color',
            'border-bottom-color',
            'border-left-color',
            'border-spacing',
            'border-style',
            'border-top-style',
            'border-right-style',
            'border-bottom-style',
            'border-left-style',
            'border-width',
            'border-top-width',
            'border-right-width',
            'border-bottom-width',
            'border-left-width',
            'border-radius',
            'border-top-right-radius',
            'border-bottom-right-radius',
            'border-bottom-left-radius',
            'border-top-left-radius',
            'border-radius-topright',
            'border-radius-bottomright',
            'border-radius-bottomleft',
            'border-radius-topleft',
            'content',
            'quotes',
            'outline',
            'outline-offset',
            'opacity',
            'filter',
            'visibility',
            'size',
            'zoom',
            'transform',
            'box-align',
            'box-flex',
            'box-orient',
            'box-pack',
            'box-shadow',
            'box-sizing',
            'table-layout',
            'animation',
            'animation-delay',
            'animation-duration',
            'animation-iteration-count',
            'animation-name',
            'animation-play-state',
            'animation-timing-function',
            'animation-fill-mode',
            'transition',
            'transition-delay',
            'transition-duration',
            'transition-property',
            'transition-timing-function',
            'background-clip',
            'backface-visibility',
            'resize',
            'appearance',
            'user-select',
            'interpolation-mode',
            'direction',
            'marks',
            'page',
            'set-link-source',
            'unicode-bidi',
            'speak'
        ]
    },
    ignoreFiles: ['**/*.js', '**/*.jsx', '**/*.tsx', '**/*.ts']
};
const getStylelintConfig = (variant) => {
    if (variant.includes('vue')) {
        return {
            ...stylelintConfig,
            extends: ['stylelint-config-standard', 'stylelint-config-recommended-vue']
        };
    }
    return stylelintConfig;
};

const writeTemplateFile = (otherLint, isVscode) => {
    const templateDir = path__default["default"].resolve(url.fileURLToPath((typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __filename).href : (document.currentScript && document.currentScript.src || new URL('bundle.cjs.js', document.baseURI).href))), '../../template');
    const files = fs__default["default"].readdirSync(templateDir);
    let filterResult = files;
    if (otherLint.indexOf('stylelint') === -1) {
        filterResult = filterResult.filter((file) => file !== '.stylelintignore');
    }
    if (otherLint.indexOf('commitlint') === -1) {
        filterResult = filterResult.filter((file) => file !== '.cz-config.cjs' && file !== '.commitlintrc.json');
    }
    if (!isVscode) {
        filterResult = filterResult.filter((file) => file !== '.vscode' && file !== 'settings.json');
    }
    for (const file of filterResult) {
        const targetPath = path__default["default"].join(root, file);
        copy(path__default["default"].join(templateDir, file), targetPath);
    }
};
const execHuskyCommand = (otherLint) => {
    const initGit = 'git init';
    const initHusky = 'npx husky install';
    const preCommit = `npx husky add .husky/pre-commit "npm run lint:lint-staged"`;
    if (!fs__default["default"].existsSync(path__default["default"].join(root, '.git'))) {
        child_process.exec(initGit, { cwd: root }, (error) => {
            if (error) {
                console.error(error);
                return;
            }
        });
    }
    child_process.exec(initHusky, { cwd: root }, (error) => {
        if (error) {
            console.error(error);
            return;
        }
        child_process.exec(preCommit, { cwd: root }, (error) => {
            if (error) {
                console.error(error);
                return;
            }
        });
        if (otherLint.includes('commitlint')) {
            const commitMsg = `npx husky add .husky/commit-msg 'npx --no-install commitlint --edit "$1"'`;
            child_process.exec(commitMsg, { cwd: root }, (error) => {
                if (error) {
                    console.error(error);
                    return;
                }
            });
        }
    });
};
const getPackageList = ({ otherLint, variant }) => {
    let result = [];
    if (otherLint.includes('stylelint')) {
        result = [...result, ...getStylelintPackages(variant)];
    }
    if (otherLint.includes('commitlint')) {
        result = [...result, ...commitlintPackages];
    }
    return [...commonPackages, ...eslintPackages[variant], ...result];
};
const setLint = ({ otherLint, variant, isVscode }) => {
    const packageManager = getPackageManager();
    const packageList = getPackageList({ otherLint, variant });
    generatePackageJson({ otherLint, variant });
    const commandMap = {
        npm: `npm install --save-dev ${packageList.join(' ')}`,
        yarn: `yarn add --dev ${packageList.join(' ')}`,
        pnpm: `pnpm install --save-dev ${packageList.join(' ')}`
    };
    const installCommand = commandMap[packageManager];
    const spinner = nanospinner.createSpinner('Installing packages...').start();
    child_process.exec(installCommand, { cwd: root }, (error) => {
        if (error) {
            spinner.error({
                text: kolorist.red('Failed to install packages!'),
                mark: '✖'
            });
            console.error(error);
            return;
        }
        execHuskyCommand(otherLint);
        writeTemplateFile(otherLint, isVscode);
        writeJsonFile('.eslintrc.json', getEslintConfig(variant));
        writeJsonFile('.stylelintrc.json', getStylelintConfig(variant));
        spinner.success({ text: kolorist.green('All done! 🎉'), mark: '✔' });
        console.log(kolorist.cyan('\n🔥 Reload your editor to activate the settings!'));
    });
};

async function init() {
    if (!checkPackageFile())
        return;
    const result = await askForProjectLint();
    console.log(kolorist.cyan('will integrates the default with husky and lint-stage'));
    setLint(result);
}
init();

if(typeof window !== 'undefined') {
  window._VERSION_ = '1.0.8'
}
